/*
 * Copyright (c) 2020-2021 Arm Limited. All rights reserved.
 *
 * SPDX-License-Identifier: Apache-2.0
 */
#include "CrossPlatformUtils.h"
#include "RteFsUtils.h"
#include "gtest/gtest.h"

using namespace std;


TEST(CrossPlatformUnitTests, GetEnv_Empty) {
  string value = CrossPlatformUtils::GetEnv("");
  EXPECT_TRUE(value.empty());
}

TEST(CrossPlatformUnitTests, GetEnv_Existing) {
  string value = CrossPlatformUtils::GetEnv("PATH"); // present always in our test environment
  EXPECT_FALSE(value.empty());
}

TEST(CrossPlatformUnitTests, GetEnv_NoExising) {
  string value = CrossPlatformUtils::GetEnv("DUMMY_ENV_VAR_NON_EXISTING");
  EXPECT_TRUE(value.empty());
}

TEST(CrossPlatformUnitTests, SetEnv_EmptyVar) {
  EXPECT_FALSE(CrossPlatformUtils::SetEnv("", "DUMMY_ENV_VAR_VALUE"));
}

TEST(CrossPlatformUnitTests, SetGetEnv_NonEmpty) {
  string value = "non_empty";
  EXPECT_TRUE(CrossPlatformUtils::SetEnv("DUMMY_ENV_VAR", value));
  EXPECT_EQ(CrossPlatformUtils::GetEnv("DUMMY_ENV_VAR"), value);
}

TEST(CrossPlatformUnitTests, SetGetEnv_Empty) {
  string value;
  EXPECT_TRUE(CrossPlatformUtils::SetEnv("DUMMY_ENV_VAR", value));
  EXPECT_TRUE(CrossPlatformUtils::GetEnv("DUMMY_ENV_VAR").empty());
}

TEST(CrossPlatformUnitTests, GetPackRootDir_ValidEnvSet) {
  string value = "packrootpath";
  EXPECT_TRUE(CrossPlatformUtils::SetEnv("CMSIS_PACK_ROOT", value));
  EXPECT_EQ(CrossPlatformUtils::GetEnv("CMSIS_PACK_ROOT"), value);
  EXPECT_EQ(CrossPlatformUtils::GetCMSISPackRootDir(), value);
}

TEST(CrossPlatformUnitTests, GetPackRootDir_NoEnvSet) {
  EXPECT_TRUE(CrossPlatformUtils::SetEnv("CMSIS_PACK_ROOT", ""));
  EXPECT_EQ(CrossPlatformUtils::GetCMSISPackRootDir(), CrossPlatformUtils::GetDefaultCMSISPackRootDir());
}

TEST(CrossPlatformUnitTests, GetPackRootDir_Default) {
  EXPECT_TRUE(CrossPlatformUtils::SetEnv("CMSIS_PACK_ROOT", ""));
#ifdef DEFAULT_PACKROOTDEF
  EXPECT_EQ(CrossPlatformUtils::GetCMSISPackRootDir().find((CrossPlatformUtils::GetEnv(DEFAULT_PACKROOTDEF))), 0);
  EXPECT_EQ(CrossPlatformUtils::GetDefaultCMSISPackRootDir().find((CrossPlatformUtils::GetEnv(DEFAULT_PACKROOTDEF))), 0);
#endif
  EXPECT_EQ(CrossPlatformUtils::GetCMSISPackRootDir(), CrossPlatformUtils::GetDefaultCMSISPackRootDir());
}

TEST(CrossPlatformUnitTests, GetExecutablePath) {
  std::error_code ec;
  std::string exePath = fs::canonical(CrossPlatformUtils::GetExecutablePath(ec)).generic_string();
  EXPECT_STREQ(TEST_BIN_PATH, exePath.c_str());
  EXPECT_EQ(ec.value(), 0);
}

TEST(CrossPlatformUnitTests, GetRegistryString) {

  if (CrossPlatformUtils::GetHostType() == "win") {
    EXPECT_TRUE(CrossPlatformUtils::GetRegistryString("HKEY_CURRENT_USER\\DUMMY_KEY_\\DUMMY_VAL").empty());
    EXPECT_TRUE(CrossPlatformUtils::GetRegistryString("DUMMY_KEY_\\DUMMY_VAL").empty());

    EXPECT_TRUE(!CrossPlatformUtils::GetRegistryString("HKEY_CURRENT_USER\\Environment\\Temp").empty());
    EXPECT_TRUE(!CrossPlatformUtils::GetRegistryString("Environment\\Temp").empty());

    EXPECT_EQ(CrossPlatformUtils::GetRegistryString("HKEY_LOCAL_MACHINE\\SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\CommonFilesDir"), "C:\\Program Files\\Common Files");
    EXPECT_EQ(CrossPlatformUtils::GetRegistryString("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\CommonFilesDir"), "C:\\Program Files\\Common Files");

    // fall-back to environment variable
    EXPECT_EQ(CrossPlatformUtils::GetRegistryString("PATH"), CrossPlatformUtils::GetEnv("PATH"));

  } else {
    EXPECT_EQ(CrossPlatformUtils::GetRegistryString("PATH"), CrossPlatformUtils::GetEnv("PATH"));
  }
}


// end of UnitTests.cpp
